/* Metronomos - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                   <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "metronomos.h"

static int started = -1;
static ao_device *device = NULL;

/* Open audio interface */
void
audio_open (struct metronomos *metro)
{
  int driver_id;
  ao_sample_format format;

  format.byte_format = AO_FMT_NATIVE;
  format.bits = 16;
  format.rate = metro->rate;
  format.channels = metro->channels;

  ao_initialize ();

  if (metro->wav_file)
    {
      driver_id = ao_driver_id ("wav");

      if (!
	  (device =
	   ao_open_file (driver_id, metro->wav_file, 1, &format, NULL)))
	fatal ("Error opening file %s.", metro->wav_file);


    }
  else
    {
      driver_id = ao_default_driver_id ();
      device = ao_open_live (driver_id, &format, NULL);
    }

  started = 0;
}

/* Close audio interface */
void
audio_close (struct metronomos *metro)
{
  if (started != -1)
    ao_close (device);

  started = -1;
  ao_shutdown ();
}

/* Write on audio interface: */
void
audio_write (struct metronomos *metro, void *what, size_t size)
{
  ao_play (device, what, size);
}

/* EOF */
