/* Metronomos - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                   <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "metronomos.h"

static void events_play (struct metronomos *metro, struct events *event);

/* Start the work: */
void
events_start (struct metronomos *metro)
{
  struct events *event;

  event = metro->event;

  /* All events... */
  while (event)
    {
      if (event->type != TYPE_SKIP)
	events_play (metro, event);

      event = event->next;
    }

  fprintf (stdout, "\nEnd events. Bye :)\n\n");
}

/* Work the single event: */
static void
events_play (struct metronomos *metro, struct events *event)
{
  float frequency = (float) event->speed / 60.0;
  float times;
  int division = 0;
  int click = 0;
  int type = event->type;
  int end = -1;
  int high = 0;
  float diff=0;

  fprintf (stdout, "\n\nEvent: %s\n\n", event->description);

  times = ONE_SEC / frequency;

  /* This type of events: */
  if (type == TYPE_SILENCE && !event->end_on_seconds && !event->end_on_times)
    {
      fprintf (stdout, "Sleep always...\n");
      while (1)
	play_silence (metro, ONE_SEC, metro->rate, metro->channels);

      return;
    }

  else if (type == TYPE_SILENCE && event->end_on_seconds)
    {
      fprintf (stdout, "Sleep for %d seconds...\n", event->end_on_seconds);
      play_silence (metro, ONE_SEC * event->end_on_seconds, metro->rate,
		    metro->channels);
      return;
    }

  else if (type == TYPE_SILENCE && event->end_on_times)
    {
      int s =
	times * (event->on_click ? event->on_click : 1) * event->end_on_times;
      fprintf (stdout,
	       "Sleep for %d times (Speed: %d - Click: %d): %d sec (%d usec)...\n",
	       event->end_on_times, event->speed, event->on_click,
	       s / ONE_SEC, s);
      play_silence (metro, s, metro->rate, metro->channels);
      return;
    }

  else if (type == TYPE_NORMAL && !event->end_on_seconds
	   && !event->end_on_times)
    {
      fprintf (stdout, "Play always...\n");
      end = -1;
    }

  else if (type == TYPE_NORMAL && event->end_on_times)
    {
      end = event->end_on_times * (event->on_click ? event->on_click : 1);
      fprintf (stdout,
	       "Play for %d times (Click: %d Total Click: %d): %d sec (%d usec)...\n",
	       event->end_on_times, event->on_click, end,
	       (end * (int) times) / ONE_SEC, end * (int) times);
      end--;
    }

  else if (type == TYPE_NORMAL && event->end_on_seconds)
    {
      end = (event->end_on_seconds * ONE_SEC) / times;
      fprintf (stdout,
	       "Play for %d seconds (Click: %d Total Click: %d): %d sec (%d usec)...\n",
	       event->end_on_seconds, end / event->on_click, end,
	       (end * (int) times) / ONE_SEC, end * (int) times);
      end--;
    }

  if (event->speed_end)
    {
      float f, t;
      f = (float) event->speed_end / 60.0;
      t = ONE_SEC / f;

      if (end != -1)
	diff = (times - t) / end;
    }

  /* Clicle: */
  while (1)
    {
      /* Alway high or you set a preference ? */
      if (event->on_click && (event->click || event->mask_click))
	{
	  /* Mask controller: */
	  if (event->mask_click)
	    {
	      if (event->mask_click[click] >= 'A'
		  && event->mask_click[click] <= 'Z')
		{
		  play_high (metro, end, click + 1, event->on_click,
			     division + 1);
		  high = 1;
		}
	      else
		{
		  play_low (metro, end, click + 1, event->on_click,
			    division + 1);
		  high = 0;
		}
	    }

	  /* Standard elements: */
	  else
	    {
	      if (click == event->click - 1)
		{
		  play_high (metro, end, click + 1, event->on_click,
			     division + 1);
		  high = 1;
		}
	      else
		{
		  play_low (metro, end, click + 1, event->on_click,
			    division + 1);
		  high = 0;
		}
	    }

	  if (click == event->on_click - 1)
	    {
	      click = 0;
	      division++;
	    }

	  else
	    click++;
	}

      else
	{
	  play_low (metro, end, 1, 1, 1);
	  high = 0;
	}

      /* If high, play silence - high_durate */
      if (high)
	play_silence (metro, times -
		      ((int) ((float) metro->click_high_durate * ONE_SEC)),
		      metro->rate, metro->channels);

      /* like up but with low sound */
      else
	play_silence (metro, times -
		      ((int) ((float) metro->click_low_durate * ONE_SEC)),
		      metro->rate, metro->channels);

      /* End ? */
      if (end == 0)
	break;
      else if (end != -1)
	end--;

      if (diff)
	times -= diff;
    }
}

/* EOF */
