/* Metronomos - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                   <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "metronomos.h"
#include "default.h"

static void *play_create_sin (float freq, int rate, int channels,
			      int *size_total);
static void *play_file (char *file, int *size, int *rate, int *channels);

#ifdef WORDS_BIGENDIAN
int16_t
swap (int16_t x)
{
  return ((((u_int16_t) x & 0x00ffU) << 8) |
	  (((u_int16_t) x & 0xff00U) >> 8));
}
#endif

/* Play the HIGH sound */
void
play_high (struct metronomos *metro, int total, int click, int on_click,
	   int division)
{
  if (metro->verbose)
    fprintf (stdout,
	     "\tTotal: %5.d - Division: %5.d - Click: [ %3.d / %3.d ] - play HIGH: %s\n",
	     total, division, click, on_click,
	     metro->click_high ? metro->click_high : "sine");

  else if (total >= 0)
    fprintf (stderr, "\r C: %5.d D: %5.d ********** [ %3.d / %3.d ]", total,
	     division, click, on_click);

  else
    fprintf (stderr, "\r          D: %5.d ********** [ %3.d / %3.d ]",
	     division, click, on_click);

  audio_write (metro, metro->click_high_data, metro->click_high_size);
}

/* Play the LOW sound */
void
play_low (struct metronomos *metro, int total, int click, int on_click,
	  int division)
{
  if (metro->verbose)
    fprintf (stdout,
	     "\tTotal: %5.d - Division: %5.d - Click: [ %3.d / %3.d ] - play LOW:  %s\n",
	     total, division, click, on_click,
	     metro->click_low ? metro->click_low : "sine");

  else if (total >= 0)
    fprintf (stderr, "\r C: %5.d D: %5.d *****      [ %3.d / %3.d ]", total,
	     division, click, on_click);

  else
    fprintf (stderr, "\r          D: %5.d *****      [ %3.d / %3.d ]",
	     division, click, on_click);

  audio_write (metro, metro->click_low_data, metro->click_low_size);
}

/* Upload in memory the wav file or the sin() */
void
play_read_file (struct metronomos *metro)
{
  int rate1 = 0, channels1 = 0;
  int rate2 = 0, channels2 = 0;

  if (metro->click_type == CLICK_FILE)
    {
      /* Wav high: */
      if (metro->click_high)
	{
	  fprintf (stdout, "Check click_high audio file %s...\n",
		   metro->click_high);
	  metro->click_high_data =
	    play_file (metro->click_high, &metro->click_high_size, &rate1,
		       &channels1);
	}

      /* Wav low: */
      if (metro->click_low)
	{
	  fprintf (stdout, "Check click_low audio file %s...\n",
		   metro->click_low);
	  metro->click_low_data =
	    play_file (metro->click_low, &metro->click_low_size, &rate2,
		       &channels2);
	}
    }

  /* If no high file: */
  if (!rate1)
    {
      if (metro->click_type == CLICK_SIN)
	{
	  fprintf (stdout, "Making click_high audio data (type: sin)...\n");
	  rate1 = rate2 ? rate2 : 44100;
	  channels1 = channels2 ? channels2 : 1;

	  metro->click_high_data =
	    play_create_sin (metro->click_high_sin_freq, rate1, channels1,
			     &metro->click_high_size);
	}

      else
	{
	  int i;
	  short *a;

	  fprintf (stdout,
		   "Making click_high audio data (type: defualt)...\n");
	  rate1 = 44100;
	  channels1 = 1;

	  if (!(metro->click_high_data = malloc (sizeof (default_click))))
	    fatal ("Error: memory.");

	  memcpy (metro->click_high_data, default_click,
		  sizeof (default_click));

	  a = metro->click_high_data;
	  for (i = 0; i < sizeof (default_click) / sizeof (short); i++)
	    a[i] *= 2;

	  metro->click_high_size = sizeof (default_click) / sizeof (short);
	}
    }

  /* If no low file: */
  if (!rate2)
    {
      if (metro->click_type == CLICK_SIN)
	{
	  fprintf (stdout, "Making click_low audio data (type: sin)...\n");
	  rate2 = rate1;
	  channels2 = channels1;

	  metro->click_low_data =
	    play_create_sin (metro->click_low_sin_freq, rate2, channels2,
			     &metro->click_low_size);
	}
      else
	{
	  fprintf (stdout,
		   "Making click_low audio data (type: default)...\n");
	  rate2 = 44100;
	  channels2 = 1;

	  if (!(metro->click_low_data = malloc (sizeof (default_click))))
	    fatal ("Error: memory.");

	  memcpy (metro->click_low_data, default_click,
		  sizeof (default_click));

	  metro->click_low_size = sizeof (default_click) / sizeof (short);
	}
    }

  /* Rate and channels check: */
  if (rate1 != rate2)
    fatal ("Rate of click_high and click_low are different!");
  if (channels1 != channels2)
    fatal ("Channels of click_high and click_low are different!");

  /* Find the durates of sound */
  metro->click_high_durate =
    (float) metro->click_high_size / rate1 * channels1;
  metro->click_low_durate = (float) metro->click_low_size / rate2 * channels2;

  /* Save rate and channels */
  metro->rate = rate1;
  metro->channels = channels1;
}

/* Upload a wav file: */
static void *
play_file (char *file, int *size, int *rate, int *channels)
{
  int ret;
  char buf[1024];
#ifdef WORDS_BIGENDIAN
  int16_t *buf16 = (int16_t *) buf;
#endif
  SNDFILE *sndfile;
  SF_INFO sfinfo;
  void *array = NULL;

  if (!(sndfile = sf_open (file, SFM_READ, &sfinfo)))
    fatal ("Error opening %s\n", file);

  *size = 0;

  while ((ret = sf_read_raw (sndfile, buf, sizeof (buf))))
    {
#ifdef WORDS_BIGENDIAN
      int i;
      for (i = 0; i < ret / 2; i++)
	buf16[i] = swap (buf16[i]);
#endif
      if (!array)
	{
	  if (!(array = (void *) malloc (ret)))
	    fatal ("Error: memory.");
	  memcpy (array, buf, ret);
	  *size = ret;
	}
      else
	{
	  if (!(array = (void *) realloc (array, ret + *size)))
	    fatal ("Error: memory.");
	  memcpy (array + *size, buf, ret);
	  *size += ret;
	}
    }

  *rate = (int) sfinfo.samplerate;
  *channels = (int) sfinfo.channels;

  sf_close (sndfile);

  return array;
}

/* Make silence: */
void
play_silence (struct metronomos *metro, int times, int rate, int channels)
{
  float k = 0;
  int size;
  int i;
  short *a;

  if (rate == 44100)
    k = 0.0441 * (times / 100) * channels;

  else if (rate == 22050)
    k = 0.02205 * (times / 100) * channels;

  else
    fatal ("No support for rate != 44100 or 22050!");

  size = (int) k;
  a = (short *) malloc (sizeof (short) * size);

  if (!a)
    fatal ("Error: memory.");

  /* Silence: */
  memset (a, 0, sizeof (short) * size);

  for (i = 0; i < 100; i++)
    audio_write (metro, a, sizeof (short) * size);

  free (a);
}

#define DURATE	0.02
#define FADE	0.004

/* Create the sin() */
static void *
play_create_sin (float freq, int rate, int channels, int *size_total)
{
  int size = rate * DURATE * channels;
  short *s = (short *) malloc (size * sizeof (short));
  int scale = -SHRT_MIN < SHRT_MAX ? -SHRT_MIN : SHRT_MAX;
  int i;
  double sample;

  if (!s)
    fatal ("Error: memory.");

  for (i = 0; i < size / channels; i++)
    {
      sample = sin ((double) i / rate * freq * 2 * M_PI);
      if (i < rate * FADE)
	sample *= (-cos ((double) i / (rate * DURATE) * 2 * M_PI) + 1) * 0.5;
      if (i > rate * (DURATE - FADE))
	sample *=
	  (-cos ((double) (size - i) / (rate * DURATE) * 2 * M_PI) + 1) * 0.5;

      s[i] = sample * scale;

      if (channels == 2)
	{
	  s[i + 1] = s[i];
	  i++;
	}
    }

  *size_total = size;

  return s;
}

/* Set volumes */
void
play_set_volume (struct metronomos *metro)
{
  int i;
  int size;
  short *what;

  fprintf (stdout, "Set volumes to %1.1f...\n", metro->volume);

  size = metro->click_high_size;
  what = metro->click_high_data;

  for (i = 0; i < size / sizeof (short); i++)
    what[i] *= metro->volume;

  size = metro->click_low_size;
  what = metro->click_low_data;
  for (i = 0; i < size / sizeof (short); i++)
    what[i] *= metro->volume;

}

/* EOF */
