/* Metronomos - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                   <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "metronomos.h"

static struct termios t_old, t_new;

static void
echo_off (void)
{
  tcgetattr (STDIN_FILENO, &t_old);
  t_new = t_old;

  t_new.c_lflag &= ~(ICANON | ECHO);
  tcsetattr (STDIN_FILENO, TCSADRAIN, &t_new);
}

static void
echo_on (void)
{
  tcsetattr (STDIN_FILENO, TCSADRAIN, &t_old);
}

void
tap_system (void)
{
  char ch;
  struct timeval t1, t2;
  int i = 0;

  echo_off ();

  while (1)
    {
      fprintf (stderr, "\rTAP: ");

      if (read (STDIN_FILENO, &ch, 1) != 1)
	break;

      if (ch == 'q' || ch == 'Q')
	break;

      if (!i)
	{
	  gettimeofday (&t1, NULL);
	  i++;
	}
      else
	{
	  double k;
	  gettimeofday (&t2, NULL);
	  k =
	    (((double) t2.tv_sec - (double) t1.tv_sec) * 1000000.0) +
	    ((double) t2.tv_usec - (double) t1.tv_usec);
	  fprintf (stderr, "%.2f          ",
		   (double) ((double) ONE_SEC / k) * 60);

	  memcpy (&t1, &t2, sizeof (struct timeval));
	}
    }

  echo_on ();
  fprintf (stderr, "\nBye :)\n");
}

/* EOF */
